/**
 * easySchema - easyWSDL toolbox Platform.
 * Copyright (c) 2008,  eBM Websourcing
 * All rights reserved.
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the University of California, Berkeley nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE REGENTS AND CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package org.ow2.easywsdl.schema.decorator;

import java.lang.reflect.InvocationTargetException;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import javax.xml.namespace.QName;

import org.ow2.easywsdl.schema.api.Documentation;
import org.ow2.easywsdl.schema.api.SchemaElement;
import org.ow2.easywsdl.schema.api.SchemaException;
import org.ow2.easywsdl.schema.api.XmlException;
import org.ow2.easywsdl.schema.api.absItf.AbsItfElement;
import org.ow2.easywsdl.schema.api.absItf.AbsItfChoice;


/**
 * @author Nicolas Boissel-Dallier - eBM WebSourcing
 */
public class DecoratorChoiceImpl<E extends AbsItfElement> implements Decorator {

	protected AbsItfChoice choice;
	
	private List<E> elements = new ArrayList<E>();
	
	public DecoratorChoiceImpl(final AbsItfChoice choice, Class<? extends E> eImpl) throws SchemaException{
		this.choice = choice;
		
		List<E> oldElems = choice.getElements();

		try {
			if(!oldElems.isEmpty()) {
				Iterator<E> iElem = oldElems.iterator();
				while(iElem.hasNext()){
					elements.add((E) eImpl.getConstructors()[0].newInstance(iElem.next()));
				}
			}
		} catch (IllegalArgumentException e) {
			throw new SchemaException(e);
		} catch (SecurityException e) {
			throw new SchemaException(e);
		} catch (InstantiationException e) {
			throw new SchemaException(e);
		} catch (IllegalAccessException e) {
			throw new SchemaException(e);
		} catch (InvocationTargetException e) {
			throw new SchemaException(e);
		}
	}

	public void addElement(E elmt) {
		this.elements.add(elmt);
		this.choice.addElement((AbsItfElement)((Decorator)elmt).getInternalObject());
	}
	
	public E createElement() {
		return (E) this.choice.createElement();
	}
	
	public List<E> getElements() {
		return this.elements;
	}
	
	public Documentation getDocumentation() {
		return this.choice.getDocumentation();
	}
	
	public Documentation createDocumentation() {
		return this.choice.createDocumentation();
	}

	public Map<QName, String> getOtherAttributes() throws XmlException {
		return this.choice.getOtherAttributes();
	}

	public void setDocumentation(Documentation doc) {
		this.choice.setDocumentation(doc);
	}

	public SchemaElement getInternalObject() {
		return this.choice;
	}

}
